function drawAll(dbName, xMode)
% This function is used to draw evaluation figures for all metrics 
% except flow consistency for a dataset.
% dbName: BuffaloXiph, SegTrackv2, BVDS, CamVid
% xMode: 
%   svPerVideo - number of supervoxel per VIDEO
%   svPerFrame - number of supervoxel per FRAME
% 
% Example: 
%   drawAll('BuffaloXiph', 'svPerVideo')
% 

% Load the environment; the method does not matter. 
[VS, PATH] = globals(dbName, 'GBH');

vidList = VS.vid;

if strcmp(VS.Name, 'BuffaloXiph')
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift';'NCut'};
elseif strcmp(VS.Name, 'SegTrackv2')
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift';'NCut'};
elseif strcmp(VS.Name, 'BVDS')
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift'};

    % If you would like to evaluate on BVDS three splits
%     vidList = VS.vid(VS.MoMap(:,1));
%     vidList = VS.vid(VS.MoMap(:,2));
%     vidList = VS.vid(VS.MoMap(:,3));

elseif strcmp(VS.Name, 'CamVid')
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift'};
else
    fprintf('No such dataset!'); 
    return
end

%% Draw All Other Metrics
stat{length(Methods)} = [];
for i=1:length(Methods)
    fprintf('Loading Method: %s ... \n', Methods{i});
    segEvalFold = fullfile(pwd,'..','Results',VS.Name,Methods{i},'segEvalRevision');
    stat{i} = loadMethodNorm(segEvalFold, vidList, xMode);
end

fprintf('Drawing figures ... \n');
Titles = {
    'TEX';
    'MSV';
    'EV';
    'UE3D';
    'SA3D';
    'BRD';
    };

colororder = getColorOrder();

% fig = figure;
for i=1:length(Titles)
%     subplot(2,3,i);
    
    fig = figure;
    
%     xlabel(xMode, 'fontsize', 22);
%     ylabel(Titles{i}, 'fontsize', 22);
    hold all;
    for j=1:length(stat)
        switch Methods{j}
            case 'GB'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-o', 'MarkerFaceColor', colororder(2,:), 'Color', colororder(2,:), 'LineWidth', 2);
            case 'GBH'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-o', 'Color', colororder(3,:), 'LineWidth', 2);
            case 'streamGBH'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-+', 'Color', colororder(8,:), 'LineWidth', 2);
            case 'SWA'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-^', 'Color', colororder(1,:), 'LineWidth', 2);
            case 'MeanShift'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-s', 'Color', colororder(7,:), 'LineWidth', 2);
            case 'NCut'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-hexagram', 'Color', colororder(4,:), 'LineWidth', 2);
            case 'TSP'
                plot(stat{j}(1,:), stat{j}(i+1,:), '-d', 'Color', colororder(5,:), 'LineWidth', 2);
        end
        
    end
    box on;
%     title(VS.Name, 'fontsize', 24);
    if strcmp(xMode, 'svPerVideo')
        xlim([0,2000]);
    elseif strcmp(xMode, 'svPerFrame')
        xlim([0,1500]);
    end
        
    switch Titles{i}
        case 'TEX'
%             legend(Methods, 'fontsize', 14, 'Location', 'NorthEast');
            ylim([0.0,1.0]);
        case 'MSV'
%             legend(Methods, 'fontsize', 14, 'Location', 'NorthEast');
            ylim([0,300]);
        case 'EV'
%             legend(Methods, 'fontsize', 14, 'Location', 'SouthEast');
            ylim([0.5,1.0]);
        case 'UE3D'
%             legend(Methods, 'fontsize', 14, 'Location', 'NorthEast');
            switch VS.Name
                case 'BuffaloXiph'
                    ylim([0,100]);
                case 'SegTrackv2'
                    ylim([0,30]);
                case 'BVDS'
                    ylim([0,200]);
                case 'CamVid'
                    ylim([0,100]);
            end
        case 'SA3D'
%             legend(Methods, 'fontsize', 14, 'Location', 'SouthEast');
            ylim([0.2,0.9]);
        case 'BRD'
%             legend(Methods, 'fontsize', 14, 'Location', 'NorthEast');
            ylim([0.5,3.0]);
    end

    % savepath = [VS.Name,'_',xMode,'_',Titles{i},'.pdf'];
    % export_fig(fig, savepath, '-transparent');
end
