function drawAllFlow(dbName, xMode)
% This function is used to draw flow consistency figures. 
% dbName: CeLiuFlow, MiddleburyFlow
% xMode: 
%   svPerVideo - number of supervoxel per VIDEO
%   svPerFrame - number of supervoxel per FRAME
%
% Example: 
%   drawAllFlow('MiddleburyFlow', 'svPerVideo')
% 

% Load the environment; the method does not matter. 
[VS, PATH] = globals(dbName, 'GBH');

vidList = VS.vid;

Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift';'NCut'};

%% Draw Flow Label Consistency
stat{length(Methods)} = [];
for i=1:length(Methods)
    fprintf('Loading Method: %s ... \n', Methods{i});
    segEvalFlowFold = fullfile(pwd,'..','Results',VS.Name,Methods{i},'segEvalFlow');
    stat{i} = loadMethodNorm(segEvalFlowFold, vidList, xMode);
end

fprintf('Drawing figures ... \n');
colororder = getColorOrder();
fig = figure;
% xlabel(xMode, 'fontsize', 22);
% ylabel('Label Consistency', 'fontsize', 22);
hold all;
for j=1:length(stat)
    switch Methods{j}
        case 'GB'
            plot(stat{j}(1,:), stat{j}(2,:), '-o', 'MarkerFaceColor', colororder(2,:), 'Color', colororder(2,:), 'LineWidth', 2);
        case 'GBH'
            plot(stat{j}(1,:), stat{j}(2,:), '-o', 'Color', colororder(3,:), 'LineWidth', 2);
        case 'streamGBH'
            plot(stat{j}(1,:), stat{j}(2,:), '-+', 'Color', colororder(8,:), 'LineWidth', 2);
        case 'SWA'
            plot(stat{j}(1,:), stat{j}(2,:), '-^', 'Color', colororder(1,:), 'LineWidth', 2);
        case 'MeanShift'
            plot(stat{j}(1,:), stat{j}(2,:), '-s', 'Color', colororder(7,:), 'LineWidth', 2);
        case 'NCut'
            plot(stat{j}(1,:), stat{j}(2,:), '-hexagram', 'Color', colororder(4,:), 'LineWidth', 2);
        case 'TSP'
            plot(stat{j}(1,:), stat{j}(2,:), '-d', 'Color', colororder(5,:), 'LineWidth', 2);
    end
    
end
box on;
% title(VS.Name, 'fontsize', 24);
if strcmp(xMode, 'svPerVideo')
    xlim([0,2000]);
elseif strcmp(xMode, 'svPerFrame')
    xlim([0,1500]);
end
ylim([0.5,1.0]);
% legend(Methods, 'fontsize', 14, 'Location', 'NorthEast');

% savepath = [VS.Name,'_',xMode,'_LC.pdf'];
% export_fig(fig, savepath, '-transparent');


