function statMethodAll = drawOne(dbName, xMode, Method)
% This function is used to draw evaluation figures for all metrics 
% except flow consistency for a single method on a dataset.
% dbName: BuffaloXiph, SegTrackv2, BVDS, CamVid
% xMode:
%   svPerVideo - number of supervoxel per VIDEO
%   svPerFrame - number of supervoxel per FRAME
% Method: GB, GBH, streamGBH, SWA, TSP, MeanShift, NCut 
% 
% Example: 
%   drawOne('BuffaloXiph', 'svPerVideo', 'GBH')
%

[VS, PATH] = globals(dbName, Method);

%% Draw All Other Metrics
fprintf('Loading Method: %s ... \n', PATH.methodName);
[~, statMethodAll] = loadMethodNorm(PATH.segEvalRevision, VS.vid, xMode);

fprintf('Drawing figures ... \n');
Titles = {
    'Temporal Extent';
    'Supervoxel Size Variation';
    'Explained Variation';
    '3D Undersegmentation Error';
    '3D Segmentation Accuracy';
    'Boundary Recall Distance';
    };
figure;
for i=1:length(Titles)
    subplot(2,3,i);
    hold all;
    for j=1:length(VS.vid)
        plot(statMethodAll(1,:,j), statMethodAll(i+1,:,j));
    end
    box on;
    % xlim([100,1000]);
    title(Titles{i});
end
hold off;
legend(VS.vid, 'Location', 'eastoutside', 'Interpreter', 'none');
