function evalOne(dbName, Method)
% This function is used to evaluate one method on a dataset with 
% all metrics. 
% dbName: BuffaloXiph, SegTrackv2, BVDS, CamVid
% Method: GB, GBH, streamGBH, SWA, TSP, MeanShift, NCut
% 
% Example: 
%   evalOne('BuffaloXiph', 'GBH')
% 

[VS, PATH] = globals(dbName, Method);

%% Evaluation: All Metrics
paramList = [];
for i=1:length(VS.vid)
    savefold = fullfile(PATH.segEvalRevision, VS.vid{i});
    if ~exist(savefold, 'dir')
        mkdir(savefold);
    end
    disp(VS.vid{i});
    
    matList = dir(fullfile(PATH.segMat, VS.vid{i}, '*.mat'));
    for j=1:length(matList)
        segPath = fullfile(PATH.segMat, VS.vid{i}, matList(j).name);
        savepath = fullfile(PATH.segEvalRevision, VS.vid{i}, matList(j).name);
        drawpath = fullfile('tmp', VS.vid{i}, PATH.methodName, matList(j).name(1:end-4));
        if ~exist(savepath, 'file')
            paramList(end+1).segPath = segPath;
            paramList(end).dbName = VS.Name;
            paramList(end).gtPath = fullfile(VS.GTPath, VS.vid{i});
            paramList(end).gtScale = VS.gtScale;
            paramList(end).framePath = fullfile(VS.PNGPath, VS.vid{i});
            paramList(end).frameFmt = 'png';
            paramList(end).savepath = savepath;
            paramList(end).drawpath = drawpath;
        end
    end

end
parfor i=1:length(paramList)
    if ~exist(paramList(i).savepath, 'file')
        disp(paramList(i).savepath);
        evalSvMap(paramList(i).segPath, paramList(i).dbName, paramList(i).gtPath, ...
            paramList(i).gtScale, paramList(i).framePath, paramList(i).frameFmt, ...
            paramList(i).savepath, paramList(i).drawpath);
    end
end
