function score = evalSvFlow(segPath, dbName, gtPath, savepath, drawpath)
% This function is used to evaluate a supervoxel segmentation video.

%% Load Ground-Truth Flow
fprintf('Loading Ground-Truth Flow ... \n');
switch dbName
    case 'CeLiuFlow'
        [VX, VY] = loadGTFlow_CeLiuFlow(gtPath);
    case 'MiddleburyFlow'
        [VX, VY] = loadGTFlow_MiddleburyFlow(gtPath);
    otherwise
        fprintf('Failed to load ground-truth. \n');
        return
end
gt_h = size(VX,1);
gt_w = size(VX,2);

%% Load Supervoxel Segmentation
fprintf('Loading Supervoxel Segmentation ... \n');
load(segPath); % svMap
svNum = double(max(svMap(:)));
[sv_h, sv_w, sv_f] = size(svMap);
if gt_h ~= sv_h || gt_w ~= sv_w
    tmpMap = [];
    for i=1:size(svMap,3)
        tmpMap = cat(3, tmpMap, imresize(svMap(:,:,i), [gt_h, gt_w], 'nearest'));
    end
    svMap = tmpMap;
end

%% Average Superpixel Per Frame
spNumList = zeros(sv_f,1);
for i=1:sv_f
    spNumList(i) = length(unique(svMap(:,:,i)));
end
spNum = mean(spNumList);

%% Supervoxel Evaluation Metrics
fprintf('Evaluating .. \n')
[LC, onMap] = scoreFlowConsistency(svMap, VX, VY);

%% Output Scores
fprintf('|%6s|%6s|%6s|\n', 'svNum', 'spNum', 'LC');
fprintf('|%6.2f|%6.2f|%6.2f|\n', svNum, spNum, LC);
score = [svNum; spNum; LC];
if ~isempty(savepath)
    save(savepath, 'score', '-v7.3');
end

%% Output Draw
% if ~isempty(drawpath)
%     mkdir(drawpath);
%     
%     onMap = ~logical(onMap);
%     
%     % black & white output
%     onMap = double(onMap);
%     for i=1:size(onMap,3)
%         imwrite(onMap(:,:,i), fullfile(drawpath,sprintf('%05d.bw.png',i)));        
%     end
%     
%     % color output
%     visualSegmentation(svMap, drawpath);
%     onMap = logical(onMap);
%     for i=1:size(onMap,3)
%         colorIM = imread(fullfile(drawpath, sprintf('%05d.png',i)));
%         rIM = colorIM(:,:,1); gIM = colorIM(:,:,2); bIM = colorIM(:,:,3);
%         rIM(onMap(:,:,i)) = 255;
%         gIM(onMap(:,:,i)) = 255;
%         bIM(onMap(:,:,i)) = 255;
%         colorIM = cat(3, rIM, gIM, bIM);
%         imwrite(uint8(colorIM), fullfile(drawpath,sprintf('%05d.col.png',i)));
%     end
%     
% end
