function [gtMap, gtList, gtPos] = loadGT_BVDS(IN_path, scale, savepath)
% This function is used to read BVDS ground-truth.
% IN_path: path to ground-truth folder of a video which contains a set of 
%   mat files.
% scale: the scale of ground-truth for evaluation. For example, 0.5 means 
%   half of the original resolution.
% gtMap: contains multiple uint16 3D volumes for different segmentation 
%   with >0 denoting segments in sequential order.
% gtList: array of ground-truth segment index in gtMap.
% gtPos: the frame id of ground-truth in a video

matList = dir(fullfile(IN_path, '*.mat'));

% To initialize gtMap
load(fullfile(IN_path, matList(1).name));
sampleNum = length(groundTruth);
gtIM = groundTruth{1}.Segmentation;
if nargin > 1
    gtIM = imresize(gtIM, scale, 'nearest');
end
[I_h, I_w, ~] = size(gtIM);
for i=1:sampleNum
    gtMap{i} = zeros(I_h,I_w,length(matList),'uint16');
end
gtPos = zeros(length(matList),1);

for i=1:length(matList)
    load(fullfile(IN_path, matList(i).name));
    for j=1:sampleNum
        gtIM = groundTruth{j}.Segmentation;
        if nargin > 1
             gtIM = imresize(gtIM, scale, 'nearest');
        end
        gtMap{j}(:,:,i) = gtIM;
    end
    gtPos(i) = str2double(matList(i).name(1:end-4));
end

% Get gtList
gtList{length(gtMap)} = [];
for i=1:length(gtMap)
    this_gtList = unique(gtMap{i});
    this_gtList = this_gtList(this_gtList>0);
    gtList{i} = this_gtList;
end

if nargin > 2
    save(savepath, 'gtMap', 'gtList', 'gtPos', '-v7.3');
end
