function  [gtMap, gtList, gtPos] = loadGT_BuffaloXiph(IN_path, scale, savepath)
% This function is used to read BuffaloXiph ground-truth.
% IN_path: path to ground-truth folder of a video which contains a set of 
%   png images, with each RGB color denoting a label.
% scale: the scale of ground-truth for evaluation. For example, 0.5 means 
%   half of the original resolution.
% gtMap: uint16 3D volume, >0 denoting objects in the order of gtColor.
% gtList: array of ground-truth segment index in gtMap.
% gtPos: the frame id of ground-truth in a video

gtColor = [
    0,0,0; % void
    128,0,0; % building
    0,128,0; % grass
    128,128,0; % tree
    0,0,128; % cow
    128,0,128; % horse
    0,128,128; % sheep
    128,128,128; % sky
    64,0,0; % mountain
    192,0,0; % airplane
    64,128,0; % water
    192,128,0; % face
    64,0,128; % car
    192,0,128; % bicycle
    64,128,128; % flower
    192,128,128; % sign
    0,64,0; % bird
    128,64,0; % book
    0,192,0; % chair
    128,64,128; % road
    0,192,128; % cat
    128,192,128; % dog
    64,64,0; % body
    192,64,0; % boat
    ];
gtColor = uint8(gtColor);

% To initialize gtMap
imList = dir(fullfile(IN_path, '*.png'));
im = imread(fullfile(IN_path, imList(1).name));
[I_h, I_w, ~] = size(im);
gtMap = zeros(I_h, I_w, length(imList), 'uint16');

for i=1:length(imList)
    gtIM = imread(fullfile(IN_path, imList(i).name));
    gtIM = reshape(gtIM,[],3);
    [Lia, Locb] = ismember(gtIM, gtColor, 'rows');
    if length(unique(Lia)) > 1
        fprintf('Wrong: png color frame does not match labels.\n');
        gtMap = [];
        return
    end
    gtIM = Locb;
    gtIM = reshape(gtIM,I_h,I_w);
    gtMap(:,:,i) = gtIM;
end

if nargin > 1
    if scale ~= 1
        gtMap_org = gtMap;
        gtMap =[];
        for i=1:size(gtMap_org,3)
            gtMap = cat(3, gtMap, imresize(gtMap_org(:,:,i), scale, 'nearest'));
        end
    end
end

% Get gtList
gtList = unique(gtMap);
gtList = gtList(gtList>0);

% Get gtPos
gtPos = (1:length(imList))';

if nargin > 2
    save(savepath, 'gtMap', 'gtList', 'gtPos', '-v7.3');
end
