function  [gtMap, gtList, gtPos] = loadGT_CamVid(IN_path, scale, savepath)
% This function is used to read CamVid ground-truth.
% IN_path: path to ground-truth folder of a video which contains a set of 
%   png images, with each RGB color denoting a label.
% scale: the scale of ground-truth for evaluation. For example, 0.5 means 
%   half of the original resolution.
% gtMap: uint16 3D volume with 0 denoting void pixel and >0 denoting
%   objects in the order of gtColor.
% gtList: array of ground-truth segment index in gtMap.
% gtPos: the frame id of ground-truth in a video

gtColor = [
    % void: all other colors
    128,0,0; % Building
    128,128,0; % Tree
    128,128,128; % Sky
    64,0,128; % Car
    192,128,128; % SignSymbol
    128,64,128; % Road
    64,64,0; % Pedestrian
    64,64,128; % Fence
    192,192,128; % ColumnPole
    0,0,192; % Sidewalk
    0,128,192; % Bicyclist
    ];

% To initialize gtMap
imList = dir(fullfile(IN_path, '*.png'));
im = imread(fullfile(IN_path, imList(1).name));
[I_h, I_w, ~] = size(im);
gtMap = zeros(I_h, I_w, length(imList), 'uint16');
gtPos = zeros(length(imList),1);

for i=1:length(imList)
    gtIM = imread(fullfile(IN_path, imList(i).name));
    gtIM = reshape(gtIM,[],3);
    [Lia, Locb] = ismember(gtIM, gtColor, 'rows');
    gtIM = Locb; % Note void is 0
    gtIM = reshape(gtIM,I_h,I_w);
    gtMap(:,:,i) = gtIM;
    gtPos(i) = str2double(imList(i).name(1:end-4));
end

if nargin > 1
    if scale ~= 1
        gtMap_org = gtMap;
        gtMap = [];
        for i=1:size(gtMap_org,3)
            gtMap = cat(3, gtMap, imresize(gtMap_org(:,:,i), scale, 'nearest'));
        end
    end
end

% Get gtList
gtList = unique(gtMap);
gtList = gtList(gtList>0);

if nargin > 2
    save(savepath, 'gtMap', 'gtList', 'gtPos', '-v7.3');
end
