function [gtMap, gtList, gtPos] = loadGT_SegTrackv2(IN_path, scale, savepath)
% This function is used to read SegTrackv2 ground-truth.
% IN_path: path to ground-truth folder of a video which contains a set of 
%   png images, with sub-folders denote multiple objects.
% scale: the scale of ground-truth for evaluation. For example, 0.5 means 
%   half of the original resolution.
% gtMap: uint16 3D volume with 0 denoting background pixel and >0 denoting
%   foreground objects in sequential order.
% gtList: array of ground-truth segment index in gtMap.
% gtPos: the frame id of ground-truth in a video

subFolds = listSubFold(IN_path);
if isempty(subFolds)
    fprintf('There is only 1 foregound object in %s. \n', IN_path);
    
    % To initialize gtMap
    imList = dir(fullfile(IN_path, '*.png'));
    im = imread(fullfile(IN_path, imList(1).name));
    [I_h, I_w, ~] = size(im);
    gtMap = zeros(I_h, I_w, length(imList), 'uint16');
    
    for i=1:length(imList)
        gtIM = imread(fullfile(IN_path, imList(i).name));
        % To approximate the boundries
        gtIM = rgb2gray(gtIM);
        gtIM(gtIM<128) = 0;
        gtIM(gtIM>=128) = 1;
        
        gtMap(:,:,i) = gtIM;
    end
else
    fprintf('There are %d foreground objects in %s. \n', ...
        length(subFolds), IN_path);
    
    % To initialize gtMap
    imList = dir(fullfile(IN_path, subFolds{1}, '*.png'));
    im = imread(fullfile(IN_path, subFolds{1}, imList(1).name));
    [I_h, I_w, ~] = size(im);
    gtMap = zeros(I_h, I_w, length(imList), 'uint16');
    
    for k=1:length(subFolds)
        for i=1:length(imList)
            gtIM = imread(fullfile(IN_path, subFolds{k}, imList(i).name));
            % To approximate the boundries
            gtIM = rgb2gray(gtIM);
            gtIM(gtIM<128) = 0;
            gtIM(gtIM>=128) = k;
            
            tmpIM = gtMap(:,:,i);
            tmpIM = tmpIM + uint16(gtIM);
            tmpIM(tmpIM>k) = k;
            gtMap(:,:,i) = tmpIM;
        end
    end
end

if nargin > 1
    if scale ~= 1
        gtMap_org = gtMap;
        gtMap = [];
        for i=1:size(gtMap_org,3)
            gtMap = cat(3, gtMap, imresize(gtMap_org(:,:,i), scale, 'nearest'));
        end
    end
end

% Get gtList
gtList = unique(gtMap);
gtList = gtList(gtList>0);

% Get gtPos
gtPos = (1:length(imList))';

if nargin > 2
    save(savepath, 'gtMap', 'gtList', 'gtPos', '-v7.3');
end
