function [statMethod, statMethodAll] = loadMethodNorm(segEvalFold, vidList, xMode)
% This function is used to align videos in a dataset. It finds a common
% range of supervoxels across all videos and linear interpolate the numbers
% for each video. For the final score it takes the average of interpolated
% socres of all videos.

dbScoreList = [];
for i=1:length(vidList)
    matList = dir(fullfile(segEvalFold, vidList{i}, '*.mat'));
    vidScoreMatrix = [];
    for j=1:length(matList)
        this = load(fullfile(segEvalFold, vidList{i}, matList(j).name));
        vidScoreMatrix = [vidScoreMatrix, this.score];
    end
    dbScoreList{i} = vidScoreMatrix;
end

svNumRange = zeros(length(vidList),2);
spNumRange = zeros(length(vidList),2);
for i=1:length(vidList)
    svNumRange(i,1) = min(dbScoreList{i}(1,:));
    svNumRange(i,2) = max(dbScoreList{i}(1,:));
    spNumRange(i,1) = min(dbScoreList{i}(2,:));
    spNumRange(i,2) = max(dbScoreList{i}(2,:));
    fprintf('%20s | %8.2f | %8.2f | %8.2f | %8.2f |\n', vidList{i}, ...
        svNumRange(i,1), svNumRange(i,2), spNumRange(i,1), spNumRange(i,2));
end

svNumPlotMin = max(svNumRange(:,1));
svNumPlotMax = min(svNumRange(:,2));
spNumPlotMin = max(spNumRange(:,1));
spNumPlotMax = min(spNumRange(:,2));
fprintf('Overall plottable svNum Range: %f - %f \n', svNumPlotMin, svNumPlotMax);
fprintf('Overall plottable spNum Range: %f - %f \n', spNumPlotMin, spNumPlotMax);


%% Interpolate supervoxels of a common range for each video.

statMethodAll = [];

if strcmp(xMode, 'svPerVideo')
    if svNumPlotMax-svNumPlotMin < 100
        Error('Videos cannot be aligned!\n');
        return
    else
        interPos = svNumPlotMin:100:svNumPlotMax;
    end
elseif strcmp(xMode, 'svPerFrame')
    if spNumPlotMax-spNumPlotMin < 100
        Error('Error: Videos cannot be aligned!\n');
        return
    else
        interPos = spNumPlotMin:100:spNumPlotMax;
    end
end


for k=1:length(vidList)
    vidScoreMatrix = dbScoreList{k};
    if strcmp(xMode, 'svPerVideo')
        inNumArray = vidScoreMatrix(1,:);
    elseif strcmp(xMode, 'svPerFrame')
        inNumArray = vidScoreMatrix(2,:);
    end
    inScoreMatrix = vidScoreMatrix(3:end,:);
    outScoreMatrix = interpScores(inNumArray, inScoreMatrix, interPos);
    statMethodAll = cat(3, statMethodAll, [interPos; outScoreMatrix]);
end
statMethod = mean(statMethodAll,3);




function outScoreMatrix = interpScores(inNumArray, inScoreMatrix, interPos)
% This function is used to interpolate scores. It first average of the same
% x bar scores.

svNumUnique = unique(inNumArray);
tmpScoreMatrix = [];
for i=1:length(svNumUnique)
    svNum = svNumUnique(i);
    idx = find(inNumArray == svNum);
    tmpScoreMatrix = [tmpScoreMatrix, sum(inScoreMatrix(:,idx),2)/length(idx)];
end
outScoreMatrix = interp1(svNumUnique, tmpScoreMatrix', interPos')';





