function [BR, BP] = scoreBoundaryRecallAndPrecision(svMap, gtMap, gtPos, mode, offset)
% This function is used to score 2D/3D boundary recall and precision.
% mode: '2D'/'3D'
% offset: 0 -- no pixel relax;
%       1 -- relax boundary by 1 pixel;
%       2 -- relax boundary by 2 pixel;
%       ...


% In case ground-truth is sparsely annotated
svMap = svMap(:,:,gtPos);
if size(svMap) ~= size(gtMap)
    fprintf('Error: gtMap and svMap dimension dismatch!\n');
    return
end

svBrWithin = genBoundaryWithinFrame(svMap);
gtBrWithin = genBoundaryWithinFrame(gtMap);
if offset == 0
    svBrWithin_relax = svBrWithin;
else
    svBrWithin_relax = imdilate(svBrWithin, strel('diamond',offset));
    % svBrWithin_relax = imdilate(svBrWithin, strel('disk',offset,0));
end
matchBrWithin = (svBrWithin_relax&gtBrWithin);

if strcmp(mode, '2D') || strcmp(mode, '2d')
    BR = sum(matchBrWithin(:))/sum(gtBrWithin(:));
    BP = sum(matchBrWithin(:))/sum(svBrWithin(:));
elseif strcmp(mode, '3D') || strcmp(mode, '3d')
    svBrBetween = genBoundaryBetweenFrames(svMap);
    gtBrBetween = genBoundaryBetweenFrames(gtMap);
    matchBrBetween = (svBrBetween&gtBrBetween); 
    BR = (sum(matchBrWithin(:)) + sum(matchBrBetween(:)))/(sum(gtBrWithin(:)) + sum(gtBrBetween(:)));
    BP = (sum(matchBrWithin(:)) + sum(matchBrBetween(:)))/(sum(svBrWithin(:)) + sum(svBrBetween(:)));
end

% if strcmp(mode, '2D') || strcmp(mode, '2d')
%     svBr = genBoundary(svMap, '2D');
%     gtBr = genBoundary(gtMap, '2D');
% elseif strcmp(mode, '3D') || strcmp(mode, '3d')
%     svBr = genBoundary(svMap, '3D');
%     gtBr = genBoundary(gtMap, '3D');
% else
%     fprintf('Error: Unknow mode!\n');
% end
% matchBr = (gtBr&svBr);
% BR = sum(matchBr(:))/sum(gtBr(:));
% BP = sum(matchBr(:))/sum(svBr(:));
