function score = scoreBoundaryRecallDistance(svMap, gtMap, gtPos)
% function [score, gtBrWithin] = scoreBoundaryRecallDistance(svMap, gtMap, gtPos)
% This function is used to score 2D boundary recall distance as in Chang et
% al. CVPR'13. 2D BRD computes the average distance between points on the
% ground truth boundary to a declared superpixel boundary averaged across
% frames.

% In case ground-truth is sparsely annotated
svMap = svMap(:,:,gtPos);
if size(svMap) ~= size(gtMap)
    fprintf('Error: gtMap and svMap dimension dismatch!\n');
    return
end

svBrWithin = genBoundaryWithinFrame(svMap);
% check if empty boundary in frame
for k=1:size(svBrWithin,3)
    if length(unique(svBrWithin(:,:,k))) == 1
        score = [];
        fprintf('Error: empty boundary in frame!\n');
        return
    end
end

gtBrWithin = genBoundaryWithinFrame(gtMap);
gtBrTotal = sum(gtBrWithin(:));

% Perfect Match
accumPercent = [];
matchBrWithin = (svBrWithin&gtBrWithin);
accumPercent(end+1) = sum(matchBrWithin(:))/gtBrTotal;

% Per Dilate Match
offset = 0;
while accumPercent(end) < 0.99
    offset = offset+1;
    svBrWithin_relax = imdilate(svBrWithin, strel('disk',offset));
    matchBrWithin = (svBrWithin_relax&gtBrWithin);
    accumPercent(end+1) = sum(matchBrWithin(:))/gtBrTotal;
end

% Final Scoring
matchPercent = accumPercent - [0,accumPercent(1:end-1)];
score = sum((0:offset).*matchPercent);
