function [UE, SA, onMapUE, onMapSA] = scoreUnderErrorAndSegAccuracy(svMap, gtMap, gtList, gtPos, mode)
% This function is used to score 3D Undersegmentation Error and 3D
% Segmentation Accuracy for supervoxels.

% In case ground-truth is sparsely annotated
svMap = svMap(:,:,gtPos);
if size(svMap) ~= size(gtMap)
    fprintf('Error: gtMap and svMap dimension dismatch!\n');
    return
end
svMap = cvlbmap(svMap);
svMap = uint32(svMap);

% get supervoxel size by frame
svSize = segSizeByFrame(svMap,1:max(svMap(:)));

% get ground-truth seg size by frame
gtSize = segSizeByFrame(gtMap, gtList);

if strcmp(mode, '3D') || strcmp(mode, '3d')
    svSize = sum(svSize,2);
    gtSize = sum(gtSize,2);
    gtUE = zeros(size(gtList));
    gtSA = zeros(size(gtList));
    for i=1:length(gtList)
        mask = (gtMap==gtList(i));
        svOnMask = svMap(mask);
        svOnID = unique(svOnMask);
        svOnSize = histc(svOnMask(:),svOnID);
        
        % Undersegmentation Error
        gtUE(i) = sum(svSize(svOnID));
        onMapUE = ismember(svMap, svOnID);
        
        % Segmentation Accuracy
        gtSA(i) = sum(svOnSize(svOnSize>=0.5*svSize(svOnID)));
        onMapSA = ismember(svMap.*uint32(mask), svOnID(svOnSize>=0.5*svSize(svOnID)));
        
        
    end
    UE = mean((gtUE-gtSize)./gtSize);
    SA = mean(gtSA./gtSize);
elseif strcmp(mode, '2D') || strcmp(mode, '2d')
    frameNum = size(gtMap,3);
    gtUE = zeros(length(gtList),frameNum);
    gtSA = zeros(length(gtList),frameNum);
    for i=1:frameNum
        thisGtMap = gtMap(:,:,i);
        thisSvMap = svMap(:,:,i);
        GtOn = unique(thisGtMap);
        for j=1:length(gtList)
            if ~ismember(gtList(j), GtOn)
                continue
            end
            mask = (thisGtMap==gtList(j));
            svOnMask = thisSvMap(mask);
            svOnID = unique(svOnMask);
            svOnSize = histc(svOnMask(:),svOnID);
            % Undersegmentation Error
            gtUE(j,i) = sum(svSize(svOnID,i));
            % Segmentation Accuracy
            gtSA(j,i) = sum(svOnSize(svOnSize>=0.5*svSize(svOnID,i)));
        end
    end
    gtSize_mask = (gtSize+(gtSize==0));
    UE = mean(sum((gtUE - gtSize)./gtSize_mask,2)./sum(gtSize>0,2));
    SA = mean(sum(gtSA./gtSize_mask,2)./sum(gtSize>0,2));
else
    fprintf('Error: unknown mode. \n');
end