function [VS, PATH] = globals(dbName, Method)
% This file is used to setup the system path and environment.
% Use one of the following dbNames:
%   BuffaloXiph, SegTrackv2, CamVid
%   BVDS, CeLiuFlow, MiddleburyFlow
% Use one of the following Method: 
%   GB, GBH, streamGBH, SWA
%   TSP, MeanShift, NCut

root = pwd;

%% Internal Library Paths
addpath(fullfile(root,'utility'));
addpath(fullfile(root,'segment'));
addpath(fullfile(root,'evaluation'));
addpath(fullfile(root,'visual'));


%% External Library Paths

% To Read Middlebury Ground-truth Flow File
addpath(fullfile(root,'..','external','flow-code-matlab'));

% To Save PDF Figures (Optional)
addpath(fullfile(root,'..','external','export_fig'));


%% Video Set
VS.Name = dbName;

switch VS.Name
    case 'CeLiuFlow'
        VS.PPMPath = fullfile(root,'..','Data',VS.Name,'PPMImages_Half');
        VS.PNGPath = fullfile(root,'..','Data',VS.Name,'PNGImages_Half');
    otherwise
        VS.PPMPath = fullfile(root,'..','Data',VS.Name,'PPMImages');
        VS.PNGPath = fullfile(root,'..','Data',VS.Name,'PNGImages');
end

VS.GTPath = fullfile(root,'..','Data',VS.Name,'GroundTruth');

VS = readVideoSet(VS, false);

if strcmp(VS.Name, 'BVDS')
    load(fullfile(root,'..','Data',VS.Name,'MoMap.mat'));
    VS.MoMap = logical(MoMap);
end

% The ground-truth scale used for evaluation.
switch VS.Name
    case 'CamVid'
        VS.gtScale = 0.5;
    case 'BVDS'
        VS.gtScale = 0.25;
    otherwise
        VS.gtScale = 1;
end

%% Segmentation Algorithms
% Load your supervoxel method here. Compile each method in their
% coressponding folders first. The script mainXXX.m will call XXX method to
% generate results for videos in the given dataset. 

PATH.methodName = Method;
switch Method
    case 'GB'
        PATH.segCode = fullfile(root,'methods','GBH','GBH');
    case 'GBH'
        PATH.segCode = fullfile(root,'methods','GBH','GBH');
    case 'streamGBH'
        PATH.segCode = fullfile(root,'methods','streamGBH','streamGBH');
    case 'SWA'
        PATH.segCode = fullfile(root,'methods','SWA','SWA');
    case 'TSP'
        % Download at http://people.csail.mit.edu/jchang7/code/TSP/TSP_2015-01-12.zip
        PATH.segCode = fullfile(root,'methods','TSP_2015-01-12'); 
    case 'MeanShift'
        % Download at http://people.csail.mit.edu/sparis/code/src/mean_shift_v6.zip
        PATH.segCode = fullfile(root,'methods','mean_shift_v6','mean_shift');
    case 'NCut'
        PATH.segCode = fullfile(root,'methods','NCut');
end

PATH.segMat = fullfile(root,'..','Results',VS.Name,Method,'Segments');
PATH.segEvalRevision = fullfile(root,'..','Results',VS.Name,Method,'segEvalRevision');
PATH.segEvalFlow = fullfile(root,'..','Results',VS.Name,Method,'segEvalFlow');



