function [E, A_points, B_points, num_sample] = Nystrom_video(dir_input_path, savepath)
% timer1: Sampling points and Constructing A, B
% timer2: Nystrom Computing
% timer3: K-means Clustering

% testing
% dir_input_path = 'nocountryforoldmen_ppm';
% sv_num = 50;
% 
% dir_output_path = 'output';

% setup free parameters
num_sample = 200;   % How many sample points ?
nvec = 150;   % How many eigenvectors ?
sigma_E = 20;   % Euclidean distance: small -> significant
sigma_Lab = 20; % Lab color space: small -> significant
I_w = 240;  % image width
I_h = 160;   % image height

if strcmp(dir_input_path(length(dir_input_path)), '/') > 0
    dir_input_path = dir_input_path(1:length(dir_input_path)-1);
end
dir_input = dir(dir_input_path);

% Read frame images
disp('Reading frame images');
num_frame = size(dir_input,1) - 2;
for i=1:num_frame
    path_image = sprintf('%s%s%05d%s',dir_input_path,'/',i,'.ppm');
    im_RGB = imread(path_image);
    im_RGB = imresize(im_RGB, [I_h I_w]);
    % im_RGB_seq(:,:,:,i) = im_RGB;
    % figure;
    % imshow(im_RGB);
    im_Lab_seq(:,:,:,i) = RGB2Lab(im_RGB);
end
im_Lab_seq = double(im_Lab_seq);
I_h = size(im_Lab_seq, 1);
I_w = size(im_Lab_seq, 2);
V_total = I_h * I_w * num_frame;

% Sampling points;
disp('Sampling points');
tic;
[A_points, B_points, num_sample] = Nystrom_sampling(V_total, num_sample, I_w, I_h, num_frame);
time = toc

% Matrix A
disp('Construct Matrix A');
tic;
A = zeros(num_sample, num_sample);
for i=1:num_sample
    [y_i, x_i, t_i] = V_map_3D(A_points(i), I_w, I_h);
    for j=1:num_sample
        if i==j
            A(i,j) = 1;
        else
            [y_j, x_j, t_j] = V_map_3D(A_points(j), I_w, I_h);
            E_distance = (x_i-x_j)^2 + (y_i-y_j)^2 + (t_i-t_j)^2;
            Lab_distance = (im_Lab_seq(y_i,x_i,1,t_i) - im_Lab_seq(y_j,x_j,1,t_j))^2 + ...
                (im_Lab_seq(y_i,x_i,2,t_i) - im_Lab_seq(y_j,x_j,2,t_j))^2 + ...
                (im_Lab_seq(y_i,x_i,3,t_i) - im_Lab_seq(y_j,x_j,3,t_j))^2;
            A(i,j) = exp((-1/2)*((E_distance/sigma_E)+(Lab_distance/sigma_Lab)));
        end
    end
end
time = toc

% Matrix B
disp('Construct Matrix B');
tic;
B = zeros(num_sample, V_total - num_sample);
for i=1:num_sample
    [y_i, x_i, t_i] = V_map_3D(A_points(i), I_w, I_h);
    parfor j=1:V_total-num_sample
        [y_j, x_j, t_j] = V_map_3D(B_points(j), I_w, I_h);
        E_distance = (x_i-x_j)^2 + (y_i-y_j)^2 + (t_i-t_j)^2;
        Lab_distance = (im_Lab_seq(y_i,x_i,1,t_i) - im_Lab_seq(y_j,x_j,1,t_j))^2 + ...
            (im_Lab_seq(y_i,x_i,2,t_i) - im_Lab_seq(y_j,x_j,2,t_j))^2 + ...
            (im_Lab_seq(y_i,x_i,3,t_i) - im_Lab_seq(y_j,x_j,3,t_j))^2;
        B(i,j) = exp((-1/2)*((E_distance/sigma_E)+(Lab_distance/sigma_Lab)));
    end
end
timer = toc

% Nystrom Computing
disp('Nystrom Computing');
tic;
E = nystrom(A, B, nvec);
timer = toc

save(savepath, 'E', 'A_points', 'B_points', 'num_sample', '-v7.3');

% clear A B im_Lab_seq;

% % K-means Clustering
% tic;
% disp('Clustering');
% opts = statset('Display', 'iter', 'MaxIter', 200);
% IDX = kmeans(E, sv_num, 'emptyaction', 'drop', 'onlinephase', 'off', 'Options', opts, 'start', 'cluster');
% timer3 = toc

% K-means + KNN
% tic;
% disp('Sample Clustering');
% Sample_idx = randsample(1:V_total, V_total/20);
% E_sample = E(Sample_idx, :);
% opts = statset('Display', 'iter', 'MaxIter', 500);
% [~, Cluster] = kmeans(E_sample, sv_num, 'emptyaction', 'singleton','onlinephase', 'off', 'Options', opts, 'start', 'cluster');
% Group = [1:sv_num]';
% IDX = knnclassify(E, Cluster, Group);
% timer3 = toc

% % Assign Colors
% disp('Assign Colors');
% Color_sample = randsample(1:255*255*255, sv_num);
% for i=1:sv_num
%     [R(i), G(i), B(i)] = V_map_3D(Color_sample(1,i),255,255);
% end
% 
% Seg_image = zeros(I_h, I_w, 3, num_frame);
% for i=1:num_sample
%     [y_i, x_i, t_i] = V_map_3D(A_points(i), I_w, I_h);
%     Seg_image(y_i, x_i, 1, t_i) = R(IDX(i));
%     Seg_image(y_i, x_i, 2, t_i) = G(IDX(i));
%     Seg_image(y_i, x_i, 3, t_i) = B(IDX(i));
% end
% for i=num_sample+1:V_total
%     [y_i, x_i, t_i] = V_map_3D(B_points(i-num_sample), I_w, I_h);
%     Seg_image(y_i, x_i, 1, t_i) = R(IDX(i));
%     Seg_image(y_i, x_i, 2, t_i) = G(IDX(i));
%     Seg_image(y_i, x_i, 3, t_i) = B(IDX(i));
% end
% 
% % mkdir(dir_output_path);
% Seg_image = uint8(Seg_image);
% for i=1:num_frame
%     savepath = sprintf('%s%s%05d%s',dir_output_path,'/',i,'.ppm');
%     imwrite(Seg_image(:,:,:,i), savepath);
% end
% 
