function runMeanShift(VS, PATH)
% This is the function to run S. Paris's Mean Shift.
% mean_shift file_name range_sigma time_sigma space_sigma persistence_threshold
% - range_sigma: The Gaussian parameter used on the color axes. We use
%   the CIE-Lab color space. The L values range between 0 and 100. Try
%   values between 5 and 10 for this parameter.
% - time_sigma: The Gaussian parameter used on the time axis. Available
%   only when segmenting videos. Try values between 5 and 15 for this parameter.
% - space_sigma: The Gaussian parameter used on the x and y axes. Try
%   values between 4 and 256 for this parameter.
% - persistence_threshold: See our article for this parameter. Try
%   values between 0 and 5.

%% Methods: Meanshift

cmdList = [];
for i=1:length(VS.vid)
    savefold = fullfile(PATH.segMat, VS.vid{i});
    if ~exist(savefold, 'dir')
        mkdir(savefold);
    end
    config_file = svMeanShift(PATH, VS.vid{i}, fullfile(VS.PPMPath, VS.vid{i}));
    
    cmdList{end+1} = sprintf('mkdir %s/00; %s %s 5 5 4 0.00 %s/00', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/01; %s %s 5 5 4 0.10 %s/01', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/02; %s %s 5 5 4 1.00 %s/02', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/03; %s %s 5 5 4 10.00 %s/03', savefold, PATH.segCode, config_file, savefold);
    
    cmdList{end+1} = sprintf('mkdir %s/10; %s %s 3 3 3 0.00 %s/10', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/11; %s %s 3 3 3 0.10 %s/11', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/12; %s %s 3 3 3 1.00 %s/12', savefold, PATH.segCode, config_file, savefold);
    
    cmdList{end+1} = sprintf('mkdir %s/00; %s %s 5 10 4 0.00 %s/00', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/01; %s %s 5 10 4 0.02 %s/01', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/02; %s %s 5 10 4 0.04 %s/02', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/03; %s %s 5 10 4 0.08 %s/03', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/04; %s %s 5 10 4 0.10 %s/04', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/05; %s %s 5 10 4 0.20 %s/05', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/06; %s %s 5 10 4 0.40 %s/06', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/07; %s %s 5 10 4 0.80 %s/07', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/08; %s %s 5 10 4 1.00 %s/08', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/09; %s %s 5 10 4 5.00 %s/09', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/10; %s %s 5 10 4 10.00 %s/10', savefold, PATH.segCode, config_file, savefold);
    
    cmdList{end+1} = sprintf('mkdir %s/40; %s %s 3 10 2 0.00 %s/40', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/41; %s %s 3 10 2 0.02 %s/41', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/42; %s %s 3 10 2 0.04 %s/42', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/43; %s %s 3 10 2 0.08 %s/43', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/44; %s %s 3 10 2 0.10 %s/44', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/45; %s %s 3 10 2 0.20 %s/45', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/46; %s %s 3 10 2 0.40 %s/46', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/47; %s %s 3 10 2 0.80 %s/47', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/48; %s %s 3 10 2 1.00 %s/48', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/49; %s %s 3 10 2 5.00 %s/49', savefold, PATH.segCode, config_file, savefold);
    cmdList{end+1} = sprintf('mkdir %s/50; %s %s 3 10 2 10.00 %s/50', savefold, PATH.segCode, config_file, savefold);
    
end
parfor i=1:length(cmdList)
    system(cmdList{i});
end


%% Convert Single Outputs
parfor i=1:length(VS.vid)
    subFolds = listSubFold(fullfile(PATH.segMat, VS.vid{i}));
    for j=1:length(subFolds)
        segFold = fullfile(PATH.segMat, VS.vid{i}, subFolds{j});
        savepath = fullfile(PATH.segMat, VS.vid{i}, [subFolds{j},'.mat']);
        if ~exist(savepath, 'file')
            convSegSingle(segFold, 'ppm', savepath);
        end
    end
end

%% Convert Hierarchy Outputs
% for i=1:length(VS.vid)
%     savefold = fullfile(PATH.segMat, VS.vid{i});
%     convSegMeanShift(savefold, 5);
% end
