function videoRGB = visualSegmentation(svMap, savefold, scale)
% This function is used to visualize the supervoxel segmentation of a video.

if ischar(svMap)
    load(svMap);
end

svMap = cvlbmap(svMap);
[I_h, I_w, frameNum] = size(svMap);
svxNum = max(svMap(:));

p = randsample((0:256*256*256-1)', svxNum);
R = floor(p./(256*256));
G = floor((p-R*256*256)./256);
B = p-R*256*256-G*256;
colorMap = cat(2,R,G,B);

if nargin > 1
    if ~exist(savefold,'dir')
        mkdir(savefold);
    end
end

videoRGB{frameNum} = [];
for i=1:frameNum
    thisMap = svMap(:,:,i);
    thisMap = reshape(thisMap,[],1);
    
    im = colorMap(thisMap,:);
    im = reshape(im, [I_h, I_w, 3]);
    
    im = uint8(im);
    
    if nargin > 2
        im = imresize(im, scale, 'nearest');
    end
    
    if nargin > 1
        savepath = sprintf('%s/%05d.png', savefold, i);
        imwrite(im, savepath);
    end
    
    videoRGB{i} = im;
end
