function visualMontageSvx(dbName, VideoName, sampleFrames, sampleSv, savepath)
% This function is used to visualize the segmentation of a video in a
% dataset. 
% dbName: BuffaloXiph, SegTrackv2, BVDS, CamVid, CeLiuFlow, MiddleburyFlow 
% VideoName: a video name in that dataset
% sampleFrames: the number of frames that you want to see
% sampleSv: the number of supervoxels per video 
% savepath: where do you want to save the output frames
% 
% Example: 
%   visualMontageSvx('BuffaloXiph', 'garden', 20, 400, 'tmp/garden')
% 

% Load the environment; the method does not matter. 
[VS, PATH] = globals(dbName, 'GBH');

if strcmp(dbName, 'BVDS')
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift'};
elseif strcmp(dbName, 'CamVid')
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift'};
else
    Methods = {'GB';'GBH';'streamGBH';'SWA';'TSP';'MeanShift';'NCut'};
end

% Get Video Frames
frameList = dir(fullfile(VS.PNGPath, VideoName, '*.png'));

% Sample frames
frameNum = length(frameList);
if sampleFrames > frameNum
    sampleFrames = frameNum;
end
framePos = (0:sampleFrames-1).*floor(frameNum/sampleFrames)+1;

%% Add Video Frames
Montage{length(Methods)+1,sampleFrames} = [];
for i=1:sampleFrames
    Montage{1,i} = imread(fullfile(VS.PNGPath, VideoName, frameList(framePos(i)).name));
end

%% Visualize Supervoxel Video Segmentation
for i=1:length(Methods)
    [matName, matSvNum] = chooseMatBySvNum(fullfile('..','Results',VS.Name,Methods{i},'segEvalRevision',VideoName), sampleSv);
    videoRGB = visualSegmentation(fullfile('..','Results',VS.Name,Methods{i},'Segments',VideoName,matName));
    Montage(i+1,:) = videoRGB(framePos);
end

%% Output Montage
makeMontage(Montage', 10, savepath, 1);